#!/usr/bin/python3
# -*- coding:utf-8 -*-

import sys
import os
import subprocess
import logging    
import time
from smbus import SMBus
from PIL import Image,ImageDraw,ImageFont
logging.basicConfig(level=logging.DEBUG)
picdir = os.path.join(os.path.dirname(os.path.dirname(os.path.realpath(__file__))), 'OLED_1in5_i2c')
print(picdir)

address = 0x3d
bus = SMBus(5)


OLED_WIDTH   = 128  #OLED width
OLED_HEIGHT  = 128  #OLED height

class OLED_1in5(object):
    def __init__(self):
        self.width = OLED_WIDTH
        self.height = OLED_HEIGHT
    
    def module_exit(self):
        bus.close()


    """    Write register address and data     """
    def command(self, cmd):
        bus.write_byte_data(address, 0x00, cmd)
        

    def Init(self):
        self.command(0xae)     #--turn off oled panel

        self.command(0x15)     #  set column address
        self.command(0x00)     #  start column   0
        self.command(0x7f)     #  end column   127

        self.command(0x75)     #   set row address
        self.command(0x00)     #  start row   0
        self.command(0x7f)     #  end row   127

        self.command(0x81)     # set contrast control
        self.command(0x80) 

        self.command(0xa0)     # gment remap
        self.command(0x51)     #51

        self.command(0xa1)     # start line
        self.command(0x00) 

        self.command(0xa2)     # display offset
        self.command(0x00) 

        self.command(0xa4)     # rmal display
        self.command(0xa8)     # set multiplex ratio
        self.command(0x7f) 

        self.command(0xb1)     # set phase leghth
        self.command(0xf1) 

        self.command(0xb3)     # set dclk
        self.command(0x00)     #80Hz:0xc1 90Hz:0xe1   100Hz:0x00   110Hz:0x30 120Hz:0x50   130Hz:0x70     01
 
        self.command(0xab)     #
        self.command(0x01)     #

        self.command(0xb6)     # set phase leghth
        self.command(0x0f) 

        self.command(0xbe) 
        self.command(0x0f) 

        self.command(0xbc) 
        self.command(0x08) 

        self.command(0xd5) 
        self.command(0x62) 

        self.command(0xfd) 
        self.command(0x12) 

        time.sleep(0.1)
        self.command(0xAF);#--turn on oled panel
        

    def SetWindows(self, Xstart, Ystart, Xend, Yend):
        if((Xstart > self.width) or (Ystart > self.height) or
        (Xend > self.width) or (Yend > self.height)):
            return
        self.command(0x15)
        self.command(Xstart//2)
        self.command(Xend//2 - 1)

        self.command(0x75)
        self.command(Ystart)
        self.command(Yend - 1)

    def clear(self):
        _buffer = [0x00]*(self.width * self.height//2)
        self.ShowImage(_buffer)             
    
    def getbuffer(self, image):
        buf = [0xff] * ((self.width//2) * self.height)
        image_monocolor = image.convert('L')#convert
        imwidth, imheight = image_monocolor.size
        pixels = image_monocolor.load()
 
        for y in range(imheight):
            for x in range(imwidth):
                # Set the bits for the column of pixels at the current position.
                addr = (int)(x/2 + y*64)
                color = pixels[x, y] % 16
                data = buf[addr] & (~0xf0 >> (x%2)*4)
                buf[addr] &= data | ((color<<4) >> ((x%2)*4))
        return buf   

    def ShowImage(self, pBuf):
        self.SetWindows(0, 0, 128, 128)
        for i in range(0, self.height):
            for j in range(0, self.width//2):
                bus.write_byte_data(address, 0x40, pBuf[j+OLED_WIDTH//2*i])
        return
if __name__=='__main__':
    
    try:
        disp = OLED_1in5()
        logging.info("\r 1.5inch OLED ")
        disp.Init()
        logging.info("clear display")
        disp.clear()
        cmd = "hostname -I | cut -d\' \' -f1"
        IP = subprocess.check_output(cmd, shell = True )
        cmd = "top -bn2 -d 0.1 | grep %Cpu | awk 'NR==2{printf \"CPU: %.2f%%\",100-$8}'"
        CPU = subprocess.check_output(cmd, shell = True )
        cmd = "free -m | awk 'NR==2{printf \"Mem: %s/%sMB %.2f%%\", $3,$2,$3*100/$2 }'"
        MemUsage = subprocess.check_output(cmd, shell = True )
        cmd = "df -h | awk '$NF==\"/\"{printf \"Disk: %d/%dGB %s\", $3,$2,$5}'"
        Disk = subprocess.check_output(cmd, shell = True )
        cmd = "date | awk '{printf \"%s %s %s %s \", $1,$2,$3,$4}'"
        date = subprocess.check_output(cmd, shell = True )
        cmd = "cat /sys/class/thermal/thermal_zone0/temp | awk '{printf \"%.1f\", $0/1000}'"
        cput = subprocess.check_output(cmd, shell= True)

        # Create blank image for drawing.
        image1 = Image.new('L', (disp.width, disp.height), 0)
        draw = ImageDraw.Draw(image1)
        font = ImageFont.truetype(os.path.join(picdir, 'Font.ttc'), 10)
        font1 = ImageFont.truetype(os.path.join(picdir, 'Font.ttc'), 18)
        font2 = ImageFont.truetype(os.path.join(picdir, 'Font.ttc'), 24)
    
        logging.info ("draw text")
        draw.text((30, 0), u'LUCKFOX', font = font1, fill = 15)
        draw.text((0, 25), "IP: " + str(IP,'utf-8'), font = font, fill = 6)
        draw.text((0, 40), str(CPU,'utf-8'), font = font, fill = 6)
        draw.text((80, 40), str(cput,'utf-8'), font = font, fill = 6)
        draw.text((0, 55), str(MemUsage,'utf-8'),  font=font, fill=6)
        draw.text((0, 70), str(Disk,'utf-8'),  font=font, fill=6)
        draw.text((0, 85), str(date,'utf-8'),  font=font, fill=6)   
        image1 = image1.rotate(0)
        disp.ShowImage(disp.getbuffer(image1))
        time.sleep(1)
            

    except IOError as e:
        logging.info(e)
        
    except KeyboardInterrupt:    
        logging.info("ctrl + c:")
        disp.module_exit()
        exit()
        
